# 创建飞船Ship类

import pygame
from pygame.sprite import Sprite


class Ship(Sprite):
    """
        初始化飞船并设置其初始位置
    """
    def __init__(self, ai_settings, screen):
        super().__init__()
        self.screen = screen
        self.ai_settings = ai_settings
        
        # 加载飞船图像并获取其外接矩形
        self.image = pygame.image.load('images/ship.bmp')
        self.rect = self.image.get_rect()     # 获取飞船图片外接矩形（即是让其可以像矩形一样被处理）
        self.screen_rect = self.screen.get_rect()        # 获取屏幕图片外接矩形（即是让其可以像矩形一样被处理）
        
        # 将每搜新飞船放在屏幕底部中央
        self.rect.centerx = self.screen_rect.centerx  # 将飞船的centerx与屏幕的centerx捆绑在一起
        self.rect.bottom = self.screen_rect.bottom     # 将飞船的bottom与屏幕的bottom捆绑在一起

        # 在飞船的属性center中存储小数值
        self.center = float(self.rect.centerx)

        # 飞船水平移动标志
        self.moving_right = False
        self.moving_left = False

    def update(self):
        """
            根据移动标志调整飞船水平位置
        """
        # 飞船持续向右移动
        if self.moving_right and self.rect.right < self.screen_rect.right:
            self.center += self.ai_settings.ship_speed_factor

        # 飞船持续向左移动
        if self.moving_left and self.rect.left > 0:
            self.center -= self.ai_settings.ship_speed_factor

        # 根据self.center更新image_rect对象
        self.rect.centerx = self.center
        
    def blitem(self):
        """
            在指定位置绘制飞船
        """
        self.screen.blit(self.image, self.rect)

    def center_ship(self):
        """让飞船在屏幕底部居中"""
        self.ship_center = self.screen_rect.centerx
